# ------
# File:        llxcfg-unix-sockets.sh
# Description: Common vars and functions for the llxcfg-backend-daemon
#              protocol using unix-domain sockets
# Author:      Luis Garcia Gisbert <garcia_luigis@gva.es>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston MA 02110-1301 USA
# --------

# variables
declare -a GLOBAL_DATA
GLOBAL_LINE=""
GLOBAL_CMD=""
GLOBAL_RC=0
CMD_SEP="::"
CMD_SUBSEP="="
CMD_EXIT="EXIT"
CMD_ERROR="ERROR"
CMD_STD="STDOUT"

RC_LINEOK=10
RC_CMDOK=0
RC_CMDEXIT=1
READ_TIMEOUT=20
READ_ATTEMPTS=3

# functions
run_client_cmd(){
   local CMD rc rc0 TXT ARG
   CMD="$1"
   ARG="$2"
   rc=$RC_CMDOK
   if [ "$GLOBAL_CMD" = "$CMD_STD" ] ; then
      echo "${GLOBAL_DATA[0]}"
   elif [ "$GLOBAL_CMD" = "$CMD_ERROR" ] ; then
      echo "${GLOBAL_DATA[0]}" >&2
   elif [ "$GLOBAL_CMD" = "$CMD_EXIT" ] ; then
      rc=$RC_CMDEXIT
      [ "${GLOBAL_DATA[0]}" ] && GLOBAL_RC=${GLOBAL_DATA[0]}
   else
      echo "Unknow server response:\"$GLOBAL_LINE\"" >&2
   fi
   return $rc
}

null_split(){
   while [ "$1" ]; do
      echo -n "$1"
      echo -ne "\00"
      shift
   done
   return 0
}

null_argv_read(){
   local i TMPFILE
   TMPFILE="$(tempfile)"
   ARGC=$(tee "$TMPFILE" |tr -dc "\0" | wc -c)
   i=1
   ARGV[0]=""
   while [ $i -le $ARGC ] ; do
      ARGV[$i]="$(cat "$TMPFILE" |cut -f $i -d "$(echo -ne "\00")")"
      i=$(($i + 1))
   done
   rm -f "$TMPFILE"
}

srv_run_external_cmd(){
   local f CMD TMP_STD TMP_ERR rc
   rc=0
   CMD="$1"
   shift
   # uhm ... we need some black magic here:
   set -o pipefail
   null_split "$@" |$CMD 2>&1 |sed -ue "s%${CMD_SUBSEP}%${CMD_SUBSEP}${CMD_SUBSEP}%g;s%^%${CMD_SEP}${CMD_STD}${CMD_SUBSEP}%;s%$% ${CMD_SUBSEP} ${CMD_SEP}%" ||rc=$?
   set +o pipefail
   # explanation:
   #    first, null_split separates args with null character and they are piped to $CMD (with stderror redirected to stdout)
   #    ($CMD can use 'xargs -0' to read arguments)
   #    the null-separated list of arguments are piped to $CMD (with stderror redirected to stdout)
   #    then, pipe $CMD stdout to an (unbuffered) sed command to get the apropiate output format
   #    by using "set -o pipefail", we can return the exit status when "something goes wrong" (only $CMD can exit without 0)
   #    the sed command has been copied from the deprecated 'cmd_file', and is duplicated here (this was because previous
   #    version of 'srv_run_external_cmd' require a 'real command' due to mispipe restrictions)
   # uff ... who said "cathedral of toothpicks" ?

   return $rc
}

read_client_LINE(){
   read GLOBAL_LINE
}

read_srv_LINE(){
   local rc rc_read num_attempts
   num_attempts=1
   rc=1
   rc_read=1
   while [ $num_attempts -le $READ_ATTEMPTS -a $rc_read -ne 0 ] ; do
      if read -t $READ_TIMEOUT GLOBAL_LINE <&6 ; then
         rc_read=0
         split_LINE || rc=$?
      else
         echo "(Frontend: read timeout, attempt $num_attempts/$READ_ATTEMPTS)" >&2
      fi
      num_attempts=$(($num_attempts +1))
   done
   return $rc
}

split_LINE(){
   local CMD_LINE DATA_TXT i
   GLOBAL_CMD=""
   GLOBAL_DATA=("")
   CMD_LINE="$(echo "$GLOBAL_LINE" |sed -ne "/^${CMD_SEP}[[:alnum:]]\+${CMD_SUBSEP}.*${CMD_SEP}$/{s%^${CMD_SEP}%%;s%${CMD_SEP}$%%;p}")"
   if [ "$CMD_LINE" ] ; then
      GLOBAL_CMD="${CMD_LINE%%$CMD_SUBSEP*}"
      DATA_TXT="${CMD_LINE#*$CMD_SUBSEP}"
      i=0
      while echo "$DATA_TXT" |grep -q " $CMD_SUBSEP " ; do
         GLOBAL_DATA[$i]=$(unescape_subsep "${DATA_TXT%% $CMD_SUBSEP *}")
         DATA_TXT="${DATA_TXT#* $CMD_SUBSEP }"    
         i=$(($i + 1))
      done
      return 0
   fi
   return 1
}
   
read_srv_data(){
   local rc LINE
   rc=$RC_CMDOK
   while [ $rc -eq $RC_CMDOK ] ; do
      read_srv_LINE || true
      run_client_cmd || rc=$?
   done
   return $GLOBAL_RC
}

unescape_subsep(){
   echo "$1" |sed -e "s%${CMD_SUBSEP}${CMD_SUBSEP}%${CMD_SUBSEP}%g"
}

escape_subsep(){
   echo "$1" |sed -e "s%${CMD_SUBSEP}%${CMD_SUBSEP}${CMD_SUBSEP}%g"
}

cmd_string(){
   local CMD="${1}${CMD_SUBSEP}"
   shift
   while [ "$1" ] ; do
      CMD="${CMD}$(escape_subsep "$1") ${CMD_SUBSEP} "
      shift
   done
   echo "${CMD_SEP}${CMD}${CMD_SEP}"
}

send_client_cmd(){
   echo "$(cmd_string "$@")"
}

send_srv_cmd(){
   local rc
   rc=0
   send_client_cmd "$@" >&7
   read_srv_data || rc=$?
   return $rc
}

cmd_file(){
   local CMD FILE_NAME
   CMD="$1"
   FILE_NAME="$2"
   if [ "$FILE_NAME" = "-" -o -s "$FILE_NAME" ] ; then
      sed -ue "s%^%${CMD_SEP}${CMD}${CMD_SUBSEP}%;s%$%${CMD_SEP}%" "$FILE_NAME"
   fi
}

exit_cmd(){
   send_client_cmd "$CMD_EXIT" "$1"
}

