#!/bin/bash
# -------
# File:        llxcfg-conffunc.sh
# Description: functions for llxcfg conffiles manipulation
#
#              Oct 2006, Luis Garcia <garcia_luigis@gva.es>
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along 
# with this program; if not, write to the Free Software Foundation, Inc., 
# 51 Franklin St, Fifth Floor, Boston MA 02110-1301 USA
# --------

PATH="/usr/sbin:/usr/bin:/sbin:/bin"
CONF_BACKUP="/var/lib/llxcfg/install-backups"

LLX_COMMONLIB="/usr/share/lliurex/llxcfg-common/llxcfg-common.sh"
[ -f "${LLX_COMMONLIB}" ] && . ${LLX_COMMONLIB}

# FUNCTIONS
error_msg(){
   echo -e "$1" >&2
   exit 1
}

get_parameter() {
   local RET_VALUE="" PARAM_NAME="$1"
   shift
   while [ "$1" ]; do
      if echo "$1" |grep -q "^${PARAM_NAME}=" ; then
         RET_VALUE="` echo "$1" |sed -ne "s%^${PARAM_NAME}=%%p"`"
      fi
      shift
   done
   echo "${RET_VALUE}"
   return 0
}

save_copy(){
   local CFG_FILE COPY_DIR MD5_FILE REL_NAME CFG_DIR DEST_DIR TMP_FILE
   CFG_FILE="`readlink -f "$1"`"
   COPY_DIR="$2"
   MD5_FILE="${COPY_DIR}.md5"
   CFG_DIR="`dirname "${CFG_FILE}"`"
   DEST_DIR="${COPY_DIR}${CFG_DIR}"
   REL_NAME="`echo "${CFG_FILE}" |sed -e 's%^/%%'`"
   [ -d "${DEST_DIR}" ] || mkdir -p "${DEST_DIR}"
   cp -a "${CFG_FILE}" "${DEST_DIR}"
   TMPFILE="`tempfile`"
   [ -f "${MD5_FILE}" ] && grep -v " ${REL_NAME}$" "${MD5_FILE}" > ${TMPFILE} || true
   ( cd /; md5sum ${REL_NAME} ) >> ${TMPFILE}
   cp ${TMPFILE} "${MD5_FILE}"
   rm -f ${TMPFILE}
}

canonicalize_name(){
   local TARGET TARGET_DIR TARGET_NAME
   TARGET="$1"
   # test target and canonicalize its name
   if [ ! -f "${TARGET}" ] ; then
      TARGET_DIR="`dirname ${TARGET}`"
      TARGET_NAME="`basename ${TARGET}`"
      if [ "${TARGET_DIR}" ] ; then
         mkdir -p "${TARGET_DIR}"
         [ -d "${TARGET_DIR}" ] || error_msg "Error creating target directory \"${TARGET_DIR}\""
      fi
      TARGET="`readlink -f "${TARGET_DIR}"`/${TARGET_NAME}"
   else
      TARGET="`readlink -f ${TARGET}`"
   fi
   echo "${TARGET}"
}

extra_tests(){
   SFILE="$1"
   TFILE="$2"
   if [ "$TEXT_ONLY" ] ; then
      if ! file "$SFILE" |grep -q " text$\| empty$\| (no magic)$" ; then
         return 1
      fi
   fi
   return 0
}

install_file(){
   local TARGET TARGET_DIR TARGET_NAME TMPDIR TMPFILE SOURCE rc
   rc=0
   SOURCE="$1"
   # test target and canonicalize its name
   TARGET="`canonicalize_name "${2}"`"
   TARGET_DIR="`dirname ${TARGET}`"
   TARGET_NAME="`basename ${TARGET}`"

   # create a temp dir and copy/create temp file
   TMPDIR="`mktemp -d`"
   TMPFILE="${TMPDIR}/${TARGET_NAME}"
   if [ ! -f "${TARGET}" ] ; then
      if [ "${SOURCE}" = "-" ]; then
         touch "${TMPFILE}"
         [ -z "$MODE" ] && MODE=${LLXCFG_MODE}
#         [ -z "$OWNER" ] && OWNER="nobody"
         [ -z "$GROUP" ] && GROUP="${LLXCFG_GROUP}"
      else
         cp -a "${SOURCE}" "${TMPFILE}"
      fi
   else
      cp -a "${TARGET}" "${TMPFILE}"
   fi

   [ "${MODE}" ] && chmod "${MODE}" "${TMPFILE}"
   [ "${OWNER}" ] && chown "${OWNER}" "${TMPFILE}"
   [ "${GROUP}" ] && chgrp "${GROUP}" "${TMPFILE}"

   # generate tempfile
   ${ACTION_CMD} "${SOURCE}" > "${TMPFILE}" || rc=$?
   # last test
   if [ $rc -eq 0 ] ; then
      extra_tests "${TMPFILE}" "${TARGET}" || rc=$?
   fi
   # install the file
   if [ $rc -eq 0 ] ; then
      if ! files_are_equal "${TMPFILE}" "${TARGET}" ; then
         mv "${TMPFILE}" "${TARGET}" || rc=$?
         [ $rc -eq 0 ] && [ "${LOG_FILE}" ] && echo "${TARGET}" >> "${LOG_FILE}"
      else
         [ "${MODE}" ] && chmod "${MODE}" "${TARGET}"
         [ "${OWNER}" ] && chown "${OWNER}" "${TARGET}"
         [ "${GROUP}" ] && chgrp "${GROUP}" "${TARGET}"
      fi
   fi
   rm -fr "${TMPDIR}"
   return $rc
}

files_are_equal(){
   [ -f "$2" ] || return 1
   MD5_1="`md5sum "$1" |cut -f 1 -d " "`"
   MD5_2="`md5sum "$2" |cut -f 1 -d " "`"
   [ "${MD5_1}" = "${MD5_2}" ] || return 1
   return 0
}

create_logfile(){
   LOG_DIR="$(dirname "LOG_FILE")"
   mkdir -p "$LOG_DIR" &> /dev/null || true
   touch "$LOG_FILE" &> /dev/null || true
   [ -f "$LOG_FILE" ] || LOG_FILE=""
   return 0
}

llxcfg_install(){
   local SOURCE TARGET rc
   [ "${LOG_FILE}" ] &&  create_logfile
   SOURCE="$1"
   TARGET="$2"
   rc=0
   # Backup the file
   [ "${BACKUPDIR}" -a -f "${TARGET}" ] && save_copy "${TARGET}" "${BACKUPDIR}"
   # install the file
   install_file "${SOURCE}" "${TARGET}" || rc=$?
   # install the 'copy' of file
   [ $rc -eq 0 ] && [ "${COPYDIR}" ] && save_copy "${TARGET}" "${COPYDIR}"
   return $rc
}
create_tmp_dir(){
   ( unset TMPDIR; mktemp -d -p ${CONF_BACKUP} )
   return 0
}

get_relative_files(){
   local SOURCE="$1"
   ( cd "${SOURCE}"; find ./ -type f |while read f; do echo "$f" |sed -e "s%^\./%%"; done )
   return 0
}

llxcfg_install_dir(){
   local SOURCE TARGET SOURCE_LIST TMP_FILE TMP_DIR TMP_LIST BASE_NAME TARGET_NAME ID_NAME
   [ "${LOG_FILE}" ] &&  create_logfile
   SOURCE="$1"
   TARGET="`readlink -f "$2"`"
   mkdir -p ${CONF_BACKUP}
   TMP_DIR="`create_tmp_dir`"
   ID_NAME="`basename "${TMP_DIR}"`"
   TMP_LIST="${TMP_DIR}.files"
   TMP_FILE="`tempfile`"
   get_relative_files "${SOURCE}" |while read BASE_NAME; do
      TARGET_NAME="${TARGET}/${BASE_NAME}"
      echo "${TARGET_NAME}" >> ${TMP_FILE}
      # do backup
      if [ -f "${TARGET_NAME}" ] ; then
         mkdir -p "`dirname "${TMP_DIR}/${BASE_NAME}"`"
	 cp -a "${TARGET_NAME}" "${TMP_DIR}/${BASE_NAME}"
      fi
   done
   # write TARGET_LIST
   mv ${TMP_FILE} ${TMP_LIST}
   # install files
   get_relative_files "${SOURCE}" |while read BASE_NAME; do
      TARGET_NAME="${TARGET}/${BASE_NAME}"
      # install the file
      install_file "${SOURCE}/${BASE_NAME}" "${TARGET_NAME}"
   done
   rm -fr ${TMP_LIST}
   rm -fr ${TMP_DIR}
   
   return 0
}

